# VolumeLock.pm (c) 2004,2006 by Peter Watkins (peterw@tux.org) 
#
# This code is derived from code with the following copyright message
# and released under the same license terms:
#
# SliMP3 Server Copyright (C) 2001 Sean Adams, Slim Devices Inc.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2.

use strict;

package Plugins::VolumeLock::Plugin;

use Slim::Control::Command;
use Slim::Control::Request;
use Slim::Utils::Strings qw (string);
use Slim::Utils::Misc qw( msg );

use vars qw($VERSION);
$VERSION = &vlVersion();

sub strings() { 
  	return '
PLUGIN_VOLUME_LOCK
	EN	VolumeLock
	
PLUGIN_VOLUME_LOCKING
	EN	Locking audio output...
	
PLUGIN_VOLUME_IS_LOCKED
	EN	Volume level is locked

PLUGIN_VOLUME_UNLOCKING
	EN	Re-enabling volume control...

PLUGIN_VOLUME_PLAY_TO_LOCK
	EN	Press PLAY to lock audio output level

PLUGIN_VOLUME_ADD_TO_UNLOCK
	EN	Press ADD to reenable volume control
'};


# whether we need to add our callback function to the stack
my $callbackSet = 0;

# whether we're enabled (the callback remains in the function 
# stack after "disabling" the plugin, so we need to keep track of this)
my $pluginEnabled = 0;

# for 6.5+, need to keep track of original volume functions
my $originalMixerVolumeCommand;
my $originalMixerMutingCommand;

sub initPlugin {
	if ( $callbackSet == 0 ) {
		# use the cleaner addDispatch technique
		$originalMixerVolumeCommand = Slim::Control::Request::addDispatch(['mixer', 'volume', '_newvalue'],[1, 0, 0, \&VolumeLock_mixerVolumeCommand]);
		$originalMixerMutingCommand = Slim::Control::Request::addDispatch(['mixer', 'muting', '_newvalue'],[1, 0, 0, \&VolumeLock_mixerMutingCommand]);
		$callbackSet = 1; 
	}
	# note that we should act
	$pluginEnabled = 1;
}

sub shutdownPlugin {
	# we should not act
	$pluginEnabled = 0;
}

sub enabled {
	if ( substr($::VERSION,0,3) gt '6.5' ) {
		&VL_msg("This plugin has not been tested with SlimServer newer than 6.5.x\n");
	}
	# this uses the Slim::Control::Request API that's in 6.5.x
	return ($::VERSION ge '6.5');
}

sub vlVersion() {
	my $rcsVersion = '$Revision: 1.8 $';
	$rcsVersion =~ s/.*:\s*([0-9\.]*).*$/$1/;
	return $rcsVersion;
}

sub getDisplayName() {
	return 'PLUGIN_VOLUME_LOCK';
}

# I think this should probably go in the Settings menu,
# but that does not seem to be possible
#sub addMenu {
#	return "SETTINGS";
#}

sub setMode() {
	my $client = shift;
	$client->lines(\&lines);
}

sub VolumeLock_mixerVolumeCommand {
	$::d_plugins && &VL_msg("VolumeLock_mixerVolumeCommand running\n");
	my @args = @_;
	my $request = $args[0];
	my $client = $request->client();
	if ( ($pluginEnabled == 1) &&
             (defined(Slim::Utils::Prefs::clientGet($client,'volumeLockVolume'))) && 
             (Slim::Utils::Prefs::clientGet($client,'volumeLockVolume') > -1 ) ) {
		$::d_plugins && &VL_msg("this player's volume is locked\n");
		&indicateLocked($client);
		$request->setStatusDone;
		return;
	}
	# else call the original function
	$::d_plugins && &VL_msg("calling original mixer function\n");
	return &$originalMixerVolumeCommand(@args);
}

sub VolumeLock_mixerMutingCommand {
	$::d_plugins && &VL_msg("VolumeLock_mixerMutingCommand running\n");
	my @args = @_;
	my $request = $args[0];
	my $client = $request->client();
	if ( ($pluginEnabled == 1) &&
             (defined(Slim::Utils::Prefs::clientGet($client,'volumeLockVolume'))) && 
             (Slim::Utils::Prefs::clientGet($client,'volumeLockVolume') > -1 ) ) {
		$::d_plugins && &VL_msg("this player's volume is locked\n");
		&indicateLocked($client);
		$request->setStatusDone;
		return;
	}
	# else call the original function
	$::d_plugins && &VL_msg("calling original mixer muting function\n");
	return &$originalMixerMutingCommand(@args);
}

sub indicateLocked($) {
	my $client = shift;
	# tell the user that the volume is locked
	my ($line1, $line2) = (string('PLUGIN_VOLUME_LOCK'), string('PLUGIN_VOLUME_IS_LOCKED'));
	$::d_plugins && &VL_msg("volume is locked\n");
	$client->showBriefly($line1, $line2, 1, 0, 1);
}

my %functions = (
	'up' => sub  {
		my $client = shift;
		$client->bumpUp($client);
	},
	'down' => sub  {
	    my $client = shift;
		$client->bumpDown($client);
	},
	'left' => sub  {
		my $client = shift;
		Slim::Buttons::Common::popModeRight($client);
	},
	'right' => sub  {
		my $client = shift;
		$client->bumpRight($client);
	},
	'play' => sub {
		my $client = shift;
		my ($line1, $line2);
		if ( (!defined(Slim::Utils::Prefs::clientGet($client,'volumeLockVolume'))) || (Slim::Utils::Prefs::clientGet($client,'volumeLockVolume') < 0) ) {
			# note the current volume as the volume to lock at
			Slim::Utils::Prefs::clientSet($client,'volumeLockVolume',Slim::Utils::Prefs::clientGet($client, "volume")); 
			# note what we're doing
			($line1, $line2) = (string('PLUGIN_VOLUME_LOCK'), string('PLUGIN_VOLUME_LOCKING'));
			$client->showBriefly($line1, $line2, 1, 0, 1);
			# refresh the display with current change option
			$client->lines(\&lines);
			$client->update();
		}
	},
	'add' => sub {
		my $client = shift;
		my ($line1, $line2);
		if (defined(Slim::Utils::Prefs::clientGet($client,'volumeLockVolume')) ) {
			# set the client's lock volume pref negative to disable locking
			Slim::Utils::Prefs::clientSet($client,'volumeLockVolume',-1);
			# note what we're doing
			($line1, $line2) = (string('PLUGIN_VOLUME_LOCK'), string('PLUGIN_VOLUME_UNLOCKING'));
			$client->showBriefly($line1, $line2, 1, 0, 1);
			# refresh the display with current change option
			$client->lines(\&lines);
			$client->update();
		}
	}
);

sub lines {
	my $client = shift;
	my ($line1, $line2);
	$line1 = string('PLUGIN_VOLUME_LOCK');
	if (defined(Slim::Utils::Prefs::clientGet($client,'volumeLockVolume')) && (Slim::Utils::Prefs::clientGet($client,'volumeLockVolume') > -1) ) {
		$line2 = string('PLUGIN_VOLUME_ADD_TO_UNLOCK');
	} else {
		$line2 = string('PLUGIN_VOLUME_PLAY_TO_LOCK');
	}
	return ($line1, $line2);
}

sub getFunctions() {
	return \%functions;
}

sub VL_msg {
	my $message = shift;
	msg("VolumeLock: $message");
}

1;

